
-- ----------------------------------------------------------------------------------------------
-- MMonad.hs  == "More Monads"
-- JMV
-- ----------------------------------------------------------------------------------------------

module MMonad ( module MMonad, module Monad, module Maybe ) where

import Monad
import Maybe

-- ----------------------------------------------------------------------------------------------
-- Identity Monad
-- ----------------------------------------------------------------------------------------------

newtype Id a =  Id { di :: a }  deriving (Eq, Show, Read)

instance  Functor Id where
   fmap f  = Id . f . di 

instance Monad Id where
   -- return      :: a -> m a
   return  = Id
   -- >>=         :: m a -> (a -> m b) -> m b
   x >>= f  = f (di x)
   
-- ----------------------------------------------------------------------------------------------
-- Maybe equality
-- ----------------------------------------------------------------------------------------------

infixl 1 `is`
is :: (Eq a) => (Maybe a) -> a -> Bool
is a' a = maybe False (== a) a'

-- ----------------------------------------------------------------------------------------------
-- Monadic iteration/restriction/loops/collections
-- ----------------------------------------------------------------------------------------------

infixl  1 `ifIs`, `till`, `with`

with :: (MonadPlus m) => m a -> a -> m a
p `with` x = p `mplus` return x

ifIs :: (MonadPlus  m) => m a -> (a -> Bool) -> (m a)
m `ifIs` t =  m >>= f where f x = if t x then return x else mzero 

till :: (Monad m) => (m a) -> (a -> Bool) -> (m a)
m `till` t =  m >>= f  where f x = if t x then  return x else  m `till` t 

-- ----------------------------------------------------------------------------------------------
-- Collections
-- ----------------------------------------------------------------------------------------------

loop :: (MonadPlus m) => (a -> m a) -> a -> m a
loop f x =  (f x >>= loop f)  `with` x

collect :: (MonadPlus m, MonadPlus m') => (m a) ->  m (m' a)
collect p = loop f mzero  where f u = p >>= return . (with u)

-- ----------------------------------------------------------------------------------------------
-- Lifts
-- ----------------------------------------------------------------------------------------------

lift :: (Functor m) => (a -> b) -> (s -> m a) -> (s -> m b)
lift  f p = fmap f . p

lift2 :: (Functor m) => (s -> a -> b) -> (s -> m a) -> (s -> m b)
lift2 f p = \s -> fmap (f s) (p s)

-- ----------------------------------------------------------------------------------------------
-- monadic cartesian closed category ?, projections, evaluations
-- ----------------------------------------------------------------------------------------------

infixl 2 *~ , ~$ , $~

(*~) :: Monad m => m a -> m b ->  m (a,b)
p *~ q = do x <- p ; y <- q ; return (x,y)

mfst :: Monad m => (a,b) -> m a
mfst = return . fst

msnd :: Monad m => (a,b) -> m b
msnd = return . snd

($~) :: Monad m => (a -> b) -> a -> m b
($~) f =  return . (f $)

(~$) :: Monad m => m (a -> b) -> a -> m b
u ~$ x = u >>=  return .  ($ x)


