module SimpleNet (module SimpleNet) where

import Net
import Relation


class (Eq a) => Spanned a where
     s_span :: [a]
--     ok     :: a -> Bool
--     ok x = x `elem` s_span   -- default

--class (Bounded a, Enum a, Eq a) => Spanned a where
--     s_span = [minBound .. maxBound]  

data (Spanned p) =>  Mark p = Mark { marks :: p -> Int }


-- auxiliar functions ----------------------------------------------------------------------------

merge :: (Spanned p) => (Int -> Int -> Int) -> (Mark p) -> (Mark p) ->  Mark p
merge f u v = Mark w   where  w p = f (marks u p) (marks v p)

toList :: (Spanned p) => (Mark p) ->  [(p,Int)]
toList u = filter ((/=0) . snd) (map (\x -> (x, marks u x)) s_span)

fromList :: (Spanned p) => [(p,Int)] -> Mark p
fromList a = Mark (\p -> case (lookup p a) of { Nothing -> 0 ; Just y -> y })
	      
posOn :: (Spanned p) => (Mark p) -> [p]
posOn m = filter ((>0) . marks m) s_span

---------------------------------------------------------------------------------------------------

instance (Spanned p) => Eq (Mark p)  where
     (==) u v = all (\p -> (marks u p) == (marks v p)) s_span 

instance (Spanned p) => Cpo (Mark p)  where
     top     = Mark (\_ -> 1)
     u .<. v = all (\p -> (marks u p) <= (marks v p)) s_span
     ord  u  = u : (map shut s_span) where
	          shut p = Mark (\q -> let x = marks u q in if q == p then max 0 (x-1)  else x)

instance (Spanned p) => State (Mark p) where
     sminus = merge (-)
     splus  = merge (+)

instance (Spanned a) => Cpo [a] where
     top = s_span
     ord u = u : (map shut u) where
	     shut a = filter (/=a) u

instance (Spanned a) => Transition [a]

instance (Spanned p, Show p) => Show (Mark p)  where
    showsPrec _  s s' = show (toList s) ++ s'

instance (Spanned p, Read p) => Read (Mark p)  where
    readsPrec n s = [(fromList z,s')] where ((z,s'):_) = readsPrec n s

-------------------------------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------------------------------


data (Spanned a, Spanned p, State s, Transition t) => SimpleNet p a s t =
      SimpleNet { mF :: a -> Mark p , mB :: a -> Mark p }

instance (Spanned p, Spanned a)  => Net (SimpleNet p a) (Mark p) [a] where
       fwrd n = foldl h (Mark (\_ -> 0)) where h u a = u `splus` (mF n a)
       bwrd n = foldl h (Mark (\_ -> 0)) where h u a = u `splus` (mB n a)





