module TransSystem (module TransSystem) where

import ST
import Relation

class TransSystem n s t  where
    transition :: n s t -> t -> ST [] s s
    trans_set  :: n s t -> [t]
--
    path :: n s t -> [t] -> ST [] s s
    path n = foldl step idT where
	       step u t = u >> transition n t 
	       idT      = rel2st (return . id)
--
    choices ::  n s t -> [t] -> ST [] s (t,s)
    choices n = foldl step mzero  where
	       step u t = u `mplus` (fmap (\x -> (t,x)) (transition n t)) 
--
    explode :: n s t ->  ST [] s (t,s)
    explode n = choices n (trans_set n)
--
    accessibility :: (Eq s) => n s t -> Relation s
    accessibility = cover . st2rel . get snd . explode
--
    accessibles :: (Eq s) => n s t -> ST [] s s
    accessibles = rel2st . accessibility
--
    state_transition_diagram :: (Eq s) => n s t -> ST [] s (s,t,s)
    state_transition_diagram n = do s <- accessibles n ; (t,u) <-  explode n ; return (s,t,u)
--
    nolock:: n s t ->  ST [] s ()
    nolock n = explode n >> return ()
--
